INTPAT ; #######################################################
 ; CUSTOM BASIC INTERPRETER PATCH FOR USE WITH ALL OF THE
 ; ROUTINES PRESENTED IN CUSTOM TRS-80 WHICH ARE TRANS-
 ; PARENT TO BASIC.  THESE ROUTINES ARE CALLED BY THIS
 ; ROUTINE, WHICH HAS ALREADY PLACED A RETURN VALUE ON THE
 ; STACK.  ALL TRANSPARENT ROUTINES MUST EXECUTE A RETURN
 ; INSTRUCTION AS THEIR FINAL INSTRUCTION TO WORK WITH 
 ; THIS CUSTOM INTERPRETER ROUTINE.  THE COMMANDS AVAIL-
 ; ABLE WITH THIS INTERPETER ARE:
 ;	/LOAD	/SAVE	/NEW	/OPEN
 ;	/ON	/OFF	/GET	/PUT
 ;	/STEP	/MEM
 ; AND OTHER USER-DEFINED "/" COMMANDS AND ROUTINES.
 ; THIS ROUTINE HAS PUSHED THE RETURN ADDRESS (1D78) ON
 ; THE STACK.  THE ROUTINE JUMPED TO IS A PSEUDO-CALL
 ; IN THAT IT EXECUTES A "RET", THUS RETURNING TO 1D78.
 ; #######################################################
 ;
 BYTE	EQU	1D78H	;ROM READ KEYS & TOKENIZE
 ;
 ; #######################################################
 ; CHECK THAT THE BASIC STACK IS IN INTERPRETATION MODE
 ; #######################################################
 ;
 BEGIN	EX	(SP),HL		; GET SP INTO HL FOR TEST
 	LD	A,L		; GET L INTO A FOR TEST
 	CP	5BH		; IS LSB OF STACK 5B?
 	JR	NZ,NOTRDY	; NOT INTERPRETING IF NZ
 	LD	A,H		; GET H INTO A FOR TEST
 	CP	1DH		; IS MSB OF STACK 1D?
 NOTRDY	EX	(SP),HL		; RESTORE STACK TO SP
 	JP	NZ,BYTE		; IF NOT 1DH THEN TO ROM
 ;
 ; #######################################################
 ; MUST HAVE BEEN AT 1D5B FOR INTERPRETATION; THEREFORE,
 ; CHECK TO SEE IF SPECIAL SLASH (/) COMMAND THAT IS NEXT
 ; #######################################################
 ;
 	CALL	BYTE		; READ CHAR. & TOKENIZE
 	PUSH	AF		; SAVE VALUE READ
 	CP	0D0H		; IS IT "/" COMMAND?
 	JR	Z,OKSLSH	; IF SO, THEN CONTINUE
 	POP	AF		; ELSE RESTORE AF VALUE
 	DEC	HL		; PUT POINTER BACK ONE
 	JP	1D78H		; AND BACK TO NORMAL ROM
 ;
 ; #######################################################
 ; SLASH (/) HAS BEEN FOUND, THEREFORE MUST BE COMMAND
 ; #######################################################
 ;
 OKSLSH	POP	AF		; RESTORE VALUE TO AF
 	CALL	BYTE		; NEXT COMMAND IN LINE
 	JR	NZ,SAVE		; GO IF ONE IS IN PLACE
 SYNERR	JP	1997H		; ?SN ERROR IF LINE END
 ;
 ; #######################################################
 ; SINCE SLASH & NEXT CHARACTER HAS BEEN FOUND, NOW NEXT
 ; CHARACTER IN LINE MUST BE TESTED FOR VALIDITY AS USER-
 ; DEFINED COMMAND.  SEE ABOVE FOR THOSE AVAILABLE HERE.
 ; #######################################################
 ;
 SAVE	LD	DE,1D78H	; GET RETURN ADDRESS
 	PUSH	DE		; PLACE IT ON STACK
 	CP	0ADH		; ---- SAVE ----
 	JP	Z,SAVER		; GO TO SAVE ROUTINE
 	CP	0BBH		; ---- NEW ----
 	JP 	Z,RENEW		; GO TO RENEW ROUTINE
 	CP	0A2H		; ---- OPEN ----
 	JP	Z,OPENER	; GO TO OPEN ROUTINE
 	CP	0CCH		; ---- STEP ----
 	JP	Z,STPSET	; GO TO STEPPING ROUTINE
 	CP	0C8H		; ---- MEM ----
 	JP	Z,MEMSET	; GO TO MEMORY SET ROUT.
 	CP	0EAH		; ---- LOC ----
 	JP	Z,RELOC		; GO TO RELOCATION ROUT.
 	CP	0A1H		; ---- ON ----
 	JP	Z,KEYON		; GO TO DEBOUNCE ON ROUT.
 	CP	0A0H		; ---- OUT ----
 	JP	Z,KEYOFF	; KILL DEBOUNCE ROUTINE
 	CP	0A4H		; ---- GET ----
 	JP	Z,COPYIN	; GO TO READ TAPE ROUTINE
 	CP	0A5H		; ---- PUT ----
 	JP	Z,DUBBER	; GO TO WRITE TAPE ROUT.
 	JP	SYNERR		; ?SN ERROR IF UNDEFINED
 ;
 ; #######################################################
 ; THIS ROUTINE WILL NOT ASSEMBLE AS IT STANDS.  IT MUST
 ; BE APPENDED TO THE OTHER ROUTINES WHICH WILL BE USED
 ; IN CONJUNCTION WITH BASIC.  ALL THE TERMS LISTED ABOVE
 ; MUST BE DEFINED, OR ELSE THEY MUST BE DELETED FROM THE
 ; ASSEMBLY LISTING.
 ; #######################################################
 ;
 	END
